@@ -61,12 +61,12 @@ static QRinput_List *QRinput_List_newEntry(QRencodeMode mode, int size, const un
 	entry->size = size;
 	entry->data = NULL;
 	if(size > 0) {
-		entry->data = (unsigned char *)malloc(size);
+		entry->data = (unsigned char *)malloc((size_t)size);
 		if(entry->data == NULL) {
 			free(entry);
 			return NULL;
 		}
-		memcpy(entry->data, data, size);
+		memcpy(entry->data, data, (size_t)size);
 	}
 	entry->bstream = NULL;
 	entry->next = NULL;
@@ -92,12 +92,12 @@ static QRinput_List *QRinput_List_dup(QRinput_List *entry)
 
 	n->mode = entry->mode;
 	n->size = entry->size;
-	n->data = (unsigned char *)malloc(n->size);
+	n->data = (unsigned char *)malloc((size_t)n->size);
 	if(n->data == NULL) {
 		free(n);
 		return NULL;
 	}
-	memcpy(n->data, entry->data, entry->size);
+	memcpy(n->data, entry->data, (size_t)entry->size);
 	n->bstream = NULL;
 	n->next = NULL;
 
@@ -117,7 +117,7 @@ QRinput *QRinput_new2(int version, QRecLevel level)
 {
 	QRinput *input;
 
-	if(version < 0 || version > QRSPEC_VERSION_MAX || level > QR_ECLEVEL_H) {
+	if(version < 0 || version > QRSPEC_VERSION_MAX || level < 0 || level > QR_ECLEVEL_H) {
 		errno = EINVAL;
 		return NULL;
 	}
@@ -415,37 +415,37 @@ static int QRinput_encodeModeNum(QRinput_List *entry, BitStream *bstream, int ve
 
 	if(mqr) {
 		if(version > 1) {
-			ret = BitStream_appendNum(bstream, version - 1, MQRSPEC_MODEID_NUM);
+			ret = BitStream_appendNum(bstream, (size_t)(version - 1), MQRSPEC_MODEID_NUM);
 			if(ret < 0) return -1;
 		}
-		ret = BitStream_appendNum(bstream, MQRspec_lengthIndicator(QR_MODE_NUM, version), entry->size);
+		ret = BitStream_appendNum(bstream, (size_t)MQRspec_lengthIndicator(QR_MODE_NUM, version), (unsigned int)entry->size);
 		if(ret < 0) return -1;
 	} else {
 		ret = BitStream_appendNum(bstream, 4, QRSPEC_MODEID_NUM);
 		if(ret < 0) return -1;
 
-		ret = BitStream_appendNum(bstream, QRspec_lengthIndicator(QR_MODE_NUM, version), entry->size);
+		ret = BitStream_appendNum(bstream, (size_t)QRspec_lengthIndicator(QR_MODE_NUM, version), (unsigned int)entry->size);
 		if(ret < 0) return -1;
 	}
 
 	words = entry->size / 3;
 	for(i = 0; i < words; i++) {
-		val  = (entry->data[i*3  ] - '0') * 100;
-		val += (entry->data[i*3+1] - '0') * 10;
-		val += (entry->data[i*3+2] - '0');
+		val  = (unsigned int)(entry->data[i*3  ] - '0') * 100;
+		val += (unsigned int)(entry->data[i*3+1] - '0') * 10;
+		val += (unsigned int)(entry->data[i*3+2] - '0');
 
 		ret = BitStream_appendNum(bstream, 10, val);
 		if(ret < 0) return -1;
 	}
 
 	if(entry->size - words * 3 == 1) {
-		val = entry->data[words*3] - '0';
+		val = (unsigned int)(entry->data[words*3] - '0');
 		ret = BitStream_appendNum(bstream, 4, val);
 		if(ret < 0) return -1;
 	} else if(entry->size - words * 3 == 2) {
-		val  = (entry->data[words*3  ] - '0') * 10;
-		val += (entry->data[words*3+1] - '0');
-		BitStream_appendNum(bstream, 7, val);
+		val  = (unsigned int)(entry->data[words*3  ] - '0') * 10;
+		val += (unsigned int)(entry->data[words*3+1] - '0');
+		ret = BitStream_appendNum(bstream, 7, val);
 		if(ret < 0) return -1;
 	}
 
@@ -521,17 +521,17 @@ static int QRinput_encodeModeAn(QRinput_List *entry, BitStream *bstream, int ver
 
 	if(mqr) {
 		if(version < 2) {
-			errno = EINVAL;
+			errno = ERANGE;
 			return -1;
 		}
-		ret = BitStream_appendNum(bstream, version - 1, MQRSPEC_MODEID_AN);
+		ret = BitStream_appendNum(bstream, (size_t)(version - 1), MQRSPEC_MODEID_AN);
 		if(ret < 0) return -1;
-		ret = BitStream_appendNum(bstream, MQRspec_lengthIndicator(QR_MODE_AN, version), entry->size);
+		ret = BitStream_appendNum(bstream, (size_t)MQRspec_lengthIndicator(QR_MODE_AN, version), (unsigned int)entry->size);
 		if(ret < 0) return -1;
 	} else {
 		ret = BitStream_appendNum(bstream, 4, QRSPEC_MODEID_AN);
 		if(ret < 0) return -1;
-		ret = BitStream_appendNum(bstream, QRspec_lengthIndicator(QR_MODE_AN, version), entry->size);
+		ret = BitStream_appendNum(bstream, (size_t)QRspec_lengthIndicator(QR_MODE_AN, version), (unsigned int)entry->size);
 		if(ret < 0) return -1;
 	}
 
@@ -583,21 +583,21 @@ static int QRinput_encodeMode8(QRinput_List *entry, BitStream *bstream, int vers
 
 	if(mqr) {
 		if(version < 3) {
-			errno = EINVAL;
+			errno = ERANGE;
 			return -1;
 		}
-		ret = BitStream_appendNum(bstream, version - 1, MQRSPEC_MODEID_8);
+		ret = BitStream_appendNum(bstream, (size_t)(version - 1), MQRSPEC_MODEID_8);
 		if(ret < 0) return -1;
-		ret = BitStream_appendNum(bstream, MQRspec_lengthIndicator(QR_MODE_8, version), entry->size);
+		ret = BitStream_appendNum(bstream, (size_t)MQRspec_lengthIndicator(QR_MODE_8, version), (unsigned int)entry->size);
 		if(ret < 0) return -1;
 	} else {
 		ret = BitStream_appendNum(bstream, 4, QRSPEC_MODEID_8);
 		if(ret < 0) return -1;
-		ret = BitStream_appendNum(bstream, QRspec_lengthIndicator(QR_MODE_8, version), entry->size);
+		ret = BitStream_appendNum(bstream, (size_t)QRspec_lengthIndicator(QR_MODE_8, version), (unsigned int)entry->size);
 		if(ret < 0) return -1;
 	}
 
-	ret = BitStream_appendBytes(bstream, entry->size, entry->data);
+	ret = BitStream_appendBytes(bstream, (size_t)entry->size, entry->data);
 	if(ret < 0) return -1;
 
 	return 0;
@@ -659,17 +659,17 @@ static int QRinput_encodeModeKanji(QRinput_List *entry, BitStream *bstream, int
 
 	if(mqr) {
 		if(version < 2) {
-			errno = EINVAL;
+			errno = ERANGE;
 			return -1;
 		}
-		ret = BitStream_appendNum(bstream, version - 1, MQRSPEC_MODEID_KANJI);
+		ret = BitStream_appendNum(bstream, (size_t)(version - 1), MQRSPEC_MODEID_KANJI);
 		if(ret < 0) return -1;
-		ret = BitStream_appendNum(bstream, MQRspec_lengthIndicator(QR_MODE_KANJI, version), entry->size/2);
+		ret = BitStream_appendNum(bstream, (size_t)MQRspec_lengthIndicator(QR_MODE_KANJI, version), (unsigned int)entry->size/2);
 		if(ret < 0) return -1;
 	} else {
 		ret = BitStream_appendNum(bstream, 4, QRSPEC_MODEID_KANJI);
 		if(ret < 0) return -1;
-		ret = BitStream_appendNum(bstream, QRspec_lengthIndicator(QR_MODE_KANJI, version), entry->size/2);
+		ret = BitStream_appendNum(bstream, (size_t)QRspec_lengthIndicator(QR_MODE_KANJI, version), (unsigned int)entry->size/2);
 		if(ret < 0) return -1;
 	}
 
@@ -715,9 +715,9 @@ static int QRinput_encodeModeStructure(QRinput_List *entry, BitStream *bstream,
 
 	ret = BitStream_appendNum(bstream, 4, QRSPEC_MODEID_STRUCTURE);
 	if(ret < 0) return -1;
-	ret = BitStream_appendNum(bstream, 4, entry->data[1] - 1);
+	ret = BitStream_appendNum(bstream, 4, entry->data[1] - 1U);
 	if(ret < 0) return -1;
-	ret = BitStream_appendNum(bstream, 4, entry->data[0] - 1);
+	ret = BitStream_appendNum(bstream, 4, entry->data[0] - 1U);
 	if(ret < 0) return -1;
 	ret = BitStream_appendNum(bstream, 8, entry->data[2]);
 	if(ret < 0) return -1;
@@ -806,7 +806,7 @@ static int QRinput_encodeModeECI(QRinput_List *entry, BitStream *bstream)
 	ret = BitStream_appendNum(bstream, 4, QRSPEC_MODEID_ECI);
 	if(ret < 0) return -1;
 
-	ret = BitStream_appendNum(bstream, words * 8, code);
+	ret = BitStream_appendNum(bstream, (size_t)words * 8, code);
 	if(ret < 0) return -1;
 
 	return 0;
@@ -890,13 +890,17 @@ static int QRinput_estimateBitStreamSizeOfEntry(QRinput_List *entry, int version
 	}
 
 	if(mqr) {
-		l = QRspec_lengthIndicator(entry->mode, version);
+		l = MQRspec_lengthIndicator(entry->mode, version);
 		m = version - 1;
 		bits += l + m;
 	} else {
 		l = QRspec_lengthIndicator(entry->mode, version);
 		m = 1 << l;
-		num = (entry->size + m - 1) / m;
+		if(entry->mode == QR_MODE_KANJI) {
+			num = (entry->size/2 + m - 1) / m;
+		} else {
+			num = (entry->size + m - 1) / m;
+		}
 
 		bits += num * (MODE_INDICATOR_SIZE + l);
 	}
@@ -927,9 +931,9 @@ STATIC_IN_RELEASE int QRinput_estimateBitStreamSize(QRinput *input, int version)
 /**
  * Estimate the required version number of the symbol.
  * @param input input data
- * @return required version number
+ * @return required version number or -1 for failure.
  */
-static int QRinput_estimateVersion(QRinput *input)
+STATIC_IN_RELEASE int QRinput_estimateVersion(QRinput *input)
 {
 	int bits;
 	int version, prev;
@@ -939,6 +943,9 @@ static int QRinput_estimateVersion(QRinput *input)
 		prev = version;
 		bits = QRinput_estimateBitStreamSize(input, prev);
 		version = QRspec_getMinimumVersion((bits + 7) / 8, input->level);
+		if(prev == 0 && version > 1) {
+			version--;
+		}
 	} while (version > prev);
 
 	return version;
@@ -1009,9 +1016,13 @@ static int QRinput_encodeBitStream(QRinput_List *entry, BitStream *bstream, int
 	QRinput_List *st1 = NULL, *st2 = NULL;
 	int prevsize;
 
-	prevsize = BitStream_size(bstream);
+	prevsize = (int)BitStream_size(bstream);
 
-	words = QRspec_maximumWords(entry->mode, version);
+	if(mqr) {
+		words = MQRspec_maximumWords(entry->mode, version);
+	} else {
+		words = QRspec_maximumWords(entry->mode, version);
+	}
 	if(words != 0 && entry->size > words) {
 		st1 = QRinput_List_newEntry(entry->mode, words, entry->data);
 		if(st1 == NULL) goto ABORT;
@@ -1055,7 +1066,7 @@ static int QRinput_encodeBitStream(QRinput_List *entry, BitStream *bstream, int
 		if(ret < 0) return -1;
 	}
 
-	return BitStream_size(bstream) - prevsize;
+	return (int)BitStream_size(bstream) - prevsize;
 ABORT:
 	QRinput_List_freeEntry(st1);
 	QRinput_List_freeEntry(st2);
@@ -1138,7 +1149,7 @@ static int QRinput_appendPaddingBit(BitStream *bstream, QRinput *input)
 	int bits, maxbits, words, maxwords, i, ret;
 	int padlen;
 
-	bits = BitStream_size(bstream);
+	bits = (int)BitStream_size(bstream);
 	maxwords = QRspec_getDataLength(input->version, input->level);
 	maxbits = maxwords * 8;
 
@@ -1151,18 +1162,18 @@ static int QRinput_appendPaddingBit(BitStream *bstream, QRinput *input)
 	}
 
 	if(maxbits - bits <= 4) {
-		return BitStream_appendNum(bstream, maxbits - bits, 0);
+		return (int)BitStream_appendNum(bstream, (size_t)(maxbits - bits), 0);
 	}
 
 	words = (bits + 4 + 7) / 8;
 
-	ret = BitStream_appendNum(bstream, words * 8 - bits, 0);
+	ret = (int)BitStream_appendNum(bstream, (size_t)(words * 8 - bits), 0);
 	if(ret < 0) return ret;
 
 	padlen = maxwords - words;
 	if(padlen > 0) {
 		for(i = 0; i < padlen; i++) {
-			ret = BitStream_appendNum(bstream, 8, (i&1)?0x11:0xec);
+			ret = (int)BitStream_appendNum(bstream, 8, (i&1)?0x11:0xec);
 			if(ret < 0) {
 				return ret;
 			}
@@ -1187,7 +1198,7 @@ static int QRinput_appendPaddingBitMQR(BitStream *bstream, QRinput *input)
 	int bits, maxbits, words, maxwords, i, ret, termbits;
 	int padlen;
 
-	bits = BitStream_size(bstream);
+	bits = (int)BitStream_size(bstream);
 	maxbits = MQRspec_getDataLengthBit(input->version, input->level);
 	maxwords = maxbits / 8;
 
@@ -1202,7 +1213,7 @@ static int QRinput_appendPaddingBitMQR(BitStream *bstream, QRinput *input)
 	termbits = input->version * 2 + 1;
 
 	if(maxbits - bits <= termbits) {
-		return BitStream_appendNum(bstream, maxbits - bits, 0);
+		return (int)BitStream_appendNum(bstream, (size_t)(maxbits - bits), 0);
 	}
 
 	bits += termbits;
@@ -1214,18 +1225,18 @@ static int QRinput_appendPaddingBitMQR(BitStream *bstream, QRinput *input)
 	} else {
 		termbits += words * 8 - bits;
 	}
-	ret = BitStream_appendNum(bstream, termbits, 0);
+	ret = (int)BitStream_appendNum(bstream, (size_t)termbits, 0);
 	if(ret < 0) return ret;
 
 	padlen = maxwords - words;
 	if(padlen > 0) {
 		for(i = 0; i < padlen; i++) {
-			ret = BitStream_appendNum(bstream, 8, (i&1)?0x11:0xec);
+			ret = (int)BitStream_appendNum(bstream, 8, (i&1)?0x11:0xec);
 			if(ret < 0) return ret;
 		}
 		termbits = maxbits - maxwords * 8;
 		if(termbits > 0) {
-			ret = BitStream_appendNum(bstream, termbits, 0);
+			ret = (int)BitStream_appendNum(bstream, (size_t)termbits, 0);
 			if(ret < 0) return ret;
 		}
 	}
@@ -1438,10 +1449,10 @@ static int QRinput_List_shrinkEntry(QRinput_List *entry, int bytes)
 {
 	unsigned char *data;
 
-	data = (unsigned char *)malloc(bytes);
+	data = (unsigned char *)malloc((size_t)bytes);
 	if(data == NULL) return -1;
 
-	memcpy(data, entry->data, bytes);
+	memcpy(data, entry->data, (size_t)bytes);
 	free(entry->data);
 	entry->data = data;
 	entry->size = bytes;
